import asyncio
import httpx
import datetime

from typing import Tuple

# Returns GMT (Greenwich Meantime)/UK time
def convert_httpx_time(httpx_timestamp: str, time_difference: int, _return: str = "Object", string_format: str = "%m-%d-%y %H:%M:%S") -> datetime.date | str:
    datetime_object = datetime.datetime.strptime(httpx_timestamp, "%a, %d %b %Y %X %Z")
    datetime_object += datetime.timedelta(seconds=time_difference)
    
    if _return == "Object":
        return datetime_object
    else:
        return datetime_object.strftime(string_format)
        
def load_unix_time(unix_timestamp: int, time_difference: int, string_format: str = '%H:%M:%S') -> str:
    datetime_object = datetime.datetime.fromtimestamp(unix_timestamp)
    
    return datetime_object.strftime(string_format)

# Only for json sent, json returned
class RequestApi:
    client = httpx.AsyncClient()

    @staticmethod
    def parse_date(response) -> str:
        try:
            return response.headers['date']
        except Exception as e:
            return str(e)

    # Always expects a json return
    @classmethod
    async def request_handler(cls, url: str, request_type: str, json_data: dict, headers: dict) -> Tuple[str, dict] | Tuple[bool, bool]:
        if request_type == "post":
            response = await cls.client.post(url, data=json_data, headers=headers)
        elif request_type == "get":
            response = await cls.client.get(url, headers=headers)
        else:
            raise Exception(f"Can't handle the request type of {request_type}")

        if response.status_code == 200:
            return cls.parse_date(response), response.json()
        else:
            return False, False
        
    def __init__(self):
        # Option for bulk request
        self.bulk_queue = asyncio.Queue()

    async def internal_parser(self, url: str, request_type: str, json_data: dict = {}, add_to_queue = False, headers: dict = {}):
        date, json_response = await self.request_handler(url, request_type, json_data, headers)

        if add_to_queue:
            await self.bulk_queue.put({"url": url, "date": date, "json_response": json_response})
    
        return date, json_response

    async def get(self, url, add_to_queue: bool = False, headers: dict = {}, _return: bool = True):
        response = await self.internal_parser(url, "get", add_to_queue=add_to_queue, headers=headers) 
        
        if _return:
            return response

    async def post(self, url, json_data: dict, add_to_queue: bool = False, headers: dict = {}, _return: bool = True):
        response = await self.internal_parser(url, "post", json_data=json_data, add_to_queue=add_to_queue, headers=headers) 

        if _return:
            return response
